//
// FlagList.h
// asommers 10-30-96
// asommers 8-5-98
//
// copyright 1998, origin systems
//

#ifndef FLAGLIST_H
#define FLAGLIST_H

class Iff;

class FlagList
{
public:

	enum
	{
		FLAGLIST_INDEX_INVALID    = -1,
		FLAGLIST_DEFAULT_SIZE = 50
	};

public :

	class Flag
	{
		friend class FlagList;

	public :

		enum 
		{
			FLAG_NAME_MAX_LENGTH  = 40
		};
		
	private :

		u_int32 flagCrc;
		u_int32 typeCrc;
		int32  value;

	private :

		Flag (const Flag&);
		Flag& operator= (const Flag&);

	private :

		Flag (void);
		~Flag (void);
	};

private :

	// Flag array
	Flag* flags;

	// Size of Flag array
	int size;

	// number of actual flags
	int numFlags;

private :

	// Adds value to flag (value can be positive or negative)
	void addToFlag ( int32 flag_num, int32 value );
	void resizeUp (void);

	FlagList& operator= (const FlagList&);

public :

	FlagList (int size=FLAGLIST_DEFAULT_SIZE);
	FlagList (Iff& iff, int num_flags);
	FlagList (const FlagList&);
	~FlagList (void);

	int getNumFlags (void) const;

	// Adds flag to the flag list
	void addFlag (u_int32 crc, u_int32 type, int32 value);

	// Returns index into flag list (for program fixups)
	// returns FLAG_INDEX_INVALID if not required
	int32 getFlagIndex (u_int32 crc, bool req=true) const;

	// Returns the value of flag
	int32 getCrc (int32 flag_num) const;
	int32 getValue (int32 flag_num) const;

	// Sets value of flag
	void setValue (int32 flag_num, int32 value);

	//
	int32 getValueFromCrc (u_int32 crc) const;
	void  setValueFromCrc (u_int32 crc, int32 value);
	void  addValueFromCrc (u_int32 crc, int32 value);

	void load (Iff& iff);
	void save (Iff& iff);

	void print (void);
};

inline int FlagList::getNumFlags (void) const
{
	return numFlags;
}

#endif
